==> brevo_contact_sync-1.0.1.tar.gz: brevo_contact_sync/src/Form/BrevoContactMappingForm.php <==
<?php

namespace Drupal\brevo_contact_sync\Form;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\sendinblue_api\Service\SendinblueApi;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines the Users Field Mapping Form.
 */
class BrevoContactMappingForm extends ConfigFormBase {

  /**
   * Drupal cache.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cache;

  /**
   * Sendinblue API service.
   *
   * @var \Drupal\sendinblue_api\Service\SendinblueApi
   */
  protected $sendinblueApi;

  /**
   * Entity Field Manager service.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $entityFieldManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->setCacheManager($container->get('cache.default'));
    $instance->setSendinblueApi($container->get('sendinblue_api'));
    $instance->setEntityFieldManager($container->get('entity_field.manager'));
    return $instance;
  }

  /**
   * Sets the cache manager dependency.
   *
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache
   *   The cache service.
   */
  protected function setCacheManager(CacheBackendInterface $cache) {
    $this->cache = $cache;
  }

  /**
   * Sets the Sendinblue API dependency.
   *
   * @param \Drupal\sendinblue_api\Service\SendinblueApi $sendinblueApi
   *   The Sendinblue API service.
   */
  protected function setSendinblueApi(SendinblueApi $sendinblueApi) {
    $this->sendinblueApi = $sendinblueApi;
  }

  /**
   * Sets the Entity Field Manager dependency.
   *
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entityFieldManager
   *   The entity field manager service.
   */
  protected function setEntityFieldManager(EntityFieldManagerInterface $entityFieldManager) {
    $this->entityFieldManager = $entityFieldManager;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'user_mapping_config_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['brevo_contact_sync.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->sendinblueApi->getConfig();
    if (empty($config['api_key'])) {
      $form['markup'] = [
        '#markup' => $this->t('You must authorize Sendinblue API before choose mapping or there are no lists available on your account.'),
      ];
      return $form;
    }
    $mappingFields = [];
    $lists = $this->sendinblueApi->getMailingLists();
    $custom_fields_data = $this->sendinblueApi->getCustomFields(FALSE);
    $custom_fields = !is_null($custom_fields_data) ? $custom_fields_data->custom_fields : FALSE;
    foreach ($custom_fields as $values) {
      $mappingFields[$values->name] = $values->name;
    }

    $options = [];
    if (!empty($lists)) {
      foreach ($lists as $list) {
        $options[$list->list_id] = $list->name;
      }
    }

    $usersFields = [];
    $listUsersFields = $this->entityFieldManager->getFieldDefinitions('user', 'user');
    $staticFields = ['name', 'uid', 'mail', 'status', 'roles'];
    foreach ($listUsersFields as $field) {
      $fieldName = $field->getName();
      if ((strpos($fieldName, 'field_') === 0) || in_array($fieldName, $staticFields)) {
        $label = $field->getLabel();
        $usersFields[$fieldName] = is_object($label) ? $label->__toString() : (string) $label;
      }
    }

    $config = $this->config('brevo_contact_sync.settings');
    $mappings = $form_state->get('mappings');
    $selected_list = $config->get('selected_list');
    if ($mappings === NULL) {
      $mappings = $config->get('mappings') ?? [
        ['user_field' => '', 'mapping_field' => '']
      ];
      $form_state->set('mappings', $mappings);
    }
    $mappingValue = ['key' => 'Key', 'value' => 'Value'];
    $form['mappings'] = [
      '#type' => 'table',
      '#header' => [
        $this->t('User Field'),
        $this->t('Value'),
        $this->t('Mapping Field'),
        $this->t('Operations'),
      ],
      '#prefix' => '<div id="mappings-wrapper">',
      '#suffix' => '</div>',
    ];

    foreach ($mappings as $index => $mapping) {
      $form['lists'] = [
        '#type' => 'select',
        '#title' => $this->t('List'),
        '#required' => TRUE,
        '#options' => $options,
        '#default_value' => $selected_list,
        '#empty_option' => $this->t('- Select a list -'),
        '#weight' => -10,
      ];
      $form['mappings'][$index]['user_field'] = [
        '#type' => 'select',
        '#options' => $usersFields,
        '#default_value' => $mapping['user_field'],
        '#title' => $this->t('User Field'),
      ];
      $form['mappings'][$index]['ref_value'] = [
        '#type' => 'select',
        '#options' => ['' => $this->t('Select value')] + $mappingValue,
        '#default_value' => $mapping['ref_value'] ?? '',
        '#title' => $this->t('Value'),
      ];
      $form['mappings'][$index]['mapping_field'] = [
        '#type' => 'select',
        '#options' => $mappingFields,
        '#default_value' => $mapping['mapping_field'],
        '#title' => $this->t('Mapping Field'),
      ];
      $form['mappings'][$index]['remove'] = [
        '#type' => 'submit',
        '#value' => $this->t('Remove'),
        '#submit' => ['::removeMapping'],
        '#name' => 'remove-' . $index,
        '#ajax' => [
          'callback' => '::ajaxCallback',
          'wrapper' => 'mappings-wrapper',
        ],
      ];
    }

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add Mapping'),
      '#submit' => ['::addMapping'],
      '#ajax' => [
        'callback' => '::ajaxCallback',
        'wrapper' => 'mappings-wrapper',
      ],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * AJAX callback to refresh the mappings table.
   */
  public function ajaxCallback(array &$form, FormStateInterface $form_state) {
    return $form['mappings'];
  }

  /**
   * Submit handler for adding a new mapping.
   */
  public function addMapping(array &$form, FormStateInterface $form_state) {
    $mappings = $form_state->get('mappings') ?? [];
    $mappings[] = ['user_field' => '', 'mapping_field' => ''];
    $form_state->set('mappings', $mappings);
    $form_state->setRebuild(TRUE);
  }

  /**
   * Submit handler for removing a mapping.
   */
  public function removeMapping(array &$form, FormStateInterface $form_state) {
    $triggering_element = $form_state->getTriggeringElement();
    $index = substr($triggering_element['#name'], 7);
    $mappings = $form_state->get('mappings');
    unset($mappings[$index]);
    $form_state->set('mappings', array_values($mappings));
    $form_state->setRebuild(TRUE);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $selected_list = $form_state->getValue('lists');
    $this->config('brevo_contact_sync.settings')
      ->set('mappings', $form_state->getValue('mappings'))
      ->set('selected_list', $selected_list)
      ->save();
    parent::submitForm($form, $form_state);
  }

}

==> brevo_contact_sync-1.0.2.tar.gz: brevo_contact_sync/src/Form/BrevoContactMappingForm.php <==
<?php

namespace Drupal\brevo_contact_sync\Form;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\sendinblue_api\Service\SendinblueApi;
use Drupal\brevo_contact_sync\Service\FieldsDefinition;
use Drupal\Core\Config\ConfigFactoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines the Users Field Mapping Form.
 */
class BrevoContactMappingForm extends ConfigFormBase {

  /**
   * Drupal cache.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cache;

  /**
   * Sendinblue API service.
   *
   * @var \Drupal\sendinblue_api\Service\SendinblueApi
   */
  protected $sendinblueApi;

  /**
   * Entity Field Manager service.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $entityFieldManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->setCacheManager($container->get('cache.default'));
    $instance->setSendinblueApi($container->get('sendinblue_api'));
    $instance->setEntityFieldManager($container->get('entity_field.manager'));
    return $instance;
  }

  /**
   * Sets the cache manager dependency.
   *
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache
   *   The cache service.
   */
  protected function setCacheManager(CacheBackendInterface $cache) {
    $this->cache = $cache;
  }

  /**
   * Sets the Sendinblue API dependency.
   *
   * @param \Drupal\sendinblue_api\Service\SendinblueApi $sendinblueApi
   *   The Sendinblue API service.
   */
  protected function setSendinblueApi(SendinblueApi $sendinblueApi) {
    $this->sendinblueApi = $sendinblueApi;
  }

  /**
   * Sets the Entity Field Manager dependency.
   *
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entityFieldManager
   *   The entity field manager service.
   */
  protected function setEntityFieldManager(EntityFieldManagerInterface $entityFieldManager) {
    $this->entityFieldManager = $entityFieldManager;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'user_mapping_config_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['brevo_contact_sync.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->sendinblueApi->getConfig();
    if (empty($config['api_key'])) {
      $form['markup'] = [
        '#markup' => $this->t('You must authorize Sendinblue API before choose mapping or there are no lists available on your account.'),
      ];
      return $form;
    }
    $mappingFields = [];
    $lists = $this->sendinblueApi->getMailingLists();
    $custom_fields_data = $this->sendinblueApi->getCustomFields(FALSE);
    $custom_fields = !is_null($custom_fields_data) ? $custom_fields_data->custom_fields : FALSE;
    foreach ($custom_fields as $values) {
      $mappingFields[$values->name] = $values->name;
    }

    $options = [];
    if (!empty($lists)) {
      foreach ($lists as $list) {
        $options[$list->list_id] = $list->name;
      }
    }

    $usersFields = [];
    $listUsersFields = $this->entityFieldManager->getFieldDefinitions('user', 'user');
    $staticFields = ['name', 'uid', 'mail', 'status', 'roles'];
    foreach ($listUsersFields as $field) {
      $fieldName = $field->getName();
      if ((strpos($fieldName, 'field_') === 0) || in_array($fieldName, $staticFields)) {
        $label = $field->getLabel();
        $usersFields[$fieldName] = is_object($label) ? $label->__toString() : (string) $label;
      }
    }

    $config = $this->config('brevo_contact_sync.settings');
    $mappings = $form_state->get('mappings');
    $selected_list = $config->get('selected_list');
    if ($mappings === NULL) {
      $mappings = $config->get('mappings') ?? [
        ['user_field' => '', 'mapping_field' => '']
      ];
      $form_state->set('mappings', $mappings);
    }
    $form['mappings'] = [
      '#type' => 'table',
      '#header' => [
        $this->t('User Field'),
        $this->t('Value'),
        $this->t('Mapping Field'),
        $this->t('Operations'),
      ],
      '#prefix' => '<div id="mappings-wrapper">',
      '#suffix' => '</div>',
    ];
    $form['lists'] = [
      '#type' => 'select',
      '#title' => $this->t('List'),
      '#required' => TRUE,
      '#options' => $options,
      '#default_value' => $selected_list,
      '#empty_option' => $this->t('- Select a list -'),
      '#weight' => -10,
    ];
    foreach ($mappings as $index => $mapping) {
      $form['mappings'][$index]['user_field'] = [
        '#type' => 'select',
        '#options' => ['' => $this->t('- Select -')] + $usersFields,
        '#default_value' => $mapping['user_field'],
        '#title' => $this->t('User Field'),
      ];
      $form['mappings'][$index]['ref_value'] = [
        '#type' => 'select', 
        '#options' => $this->fieldContent(),
        '#default_value' => $mapping['ref_value'] ?? '',
        '#title' => $this->t('Value'),
      ];
      $form['mappings'][$index]['mapping_field'] = [
        '#type' => 'select',
        '#options' => ['' => $this->t('- Select -')] + $mappingFields,
        '#default_value' => $mapping['mapping_field'],
        '#title' => $this->t('Mapping Field'),
      ];
      $disabled = false;
      if($index == 0){
        $disabled = true;
      }
      $form['mappings'][$index]['remove'] = [
        '#type' => 'submit',
        '#value' => $this->t('Remove'),
        '#submit' => ['::removeMapping'],
        '#name' => 'remove-' . $index,
        '#attributes' => [
          'disabled' => $disabled,
        ],
        '#ajax' => [
          'callback' => '::ajaxCallback',
          'wrapper' => 'mappings-wrapper',
        ],
      ];
    }

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add Mapping'),
      '#submit' => ['::addMapping'],
      '#ajax' => [
        'callback' => '::ajaxCallback',
        'wrapper' => 'mappings-wrapper',
      ],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * AJAX callback to refresh the mappings table.
   */
  public function ajaxCallback(array &$form, FormStateInterface $form_state) {
    return $form['mappings'];
  }

  public function fieldContent() {
    $options = [];
    $listUsersFields = $this->entityFieldManager->getFieldDefinitions('user', 'user');
    foreach ($listUsersFields as $field) {
      $fieldName = $field->getName();
      if (isset($fieldName)) {
        $selected_value = $fieldName;
        $entityManager = \Drupal::service('entity_field.manager');
        $definitions = $entityManager->getFieldDefinitions('user', 'user');
  
        if (isset($definitions[$selected_value])) {
          $definition = $definitions[$selected_value];
          $field_type = $definition->getType();
          switch ($field_type) {
            case 'string':
            case 'email':
            case 'integer':
            case 'decimal':
            case 'float':
            case 'boolean':
              $options['String'] = [
                'string_value' => $this->t('Value'),
              ];
            break;
            case 'list_string':
              $options['List'] = [
                'list_string_value' => $this->t('Value'),
                'list_string_key' => $this->t('Key'),
              ];
              break;
  
            case 'address':
              $options['Address'] = [
                'address_country_code' => $this->t('Country'),
                'address_given_name' => $this->t('First Name'),
                'address_family_name' => $this->t('Last Name'),
                'address_organization' => $this->t('Organization'),
                'address_address_line1' => $this->t('Address Line 1'),
                'address_address_line2' => $this->t('Address Line 2'),
                'address_address_line3' => $this->t('Address Line 3'),
                'address_locality' => $this->t('City'),
                'address_postal_code' => $this->t('Postal Code'),
                'address_administrative_area' => $this->t('Administrative area (e.g. State or Province)'),
                'address_sorting_code' => $this->t('Sorting Code'),
              ];
              break;
  
            case 'link':
              $options['Link'] = [
                'link_url' => $this->t('URL'),
                'link_label' => $this->t('Label'),
              ];
              break;
  
            case 'entity_reference':
              $options['Reference'] = [
                'entity_reference_target_id' => $this->t('Target ID'),
                'entity_reference_name' => $this->t('Label'),
              ];
              break;
            case 'image':
            case 'file':
              $options['file'] = [
                'file_uri' => $this->t('URI'),
                'file_full_url' => $this->t('Full URL'),
              ];
              break;
          }
        }
      }
    }

    return  $options;
  }

  /**
   * Submit handler for adding a new mapping.
   */
  public function addMapping(array &$form, FormStateInterface $form_state) {
    $mappings = $form_state->get('mappings') ?? [];
    $mappings[] = ['user_field' => '', 'mapping_field' => ''];
    $form_state->set('mappings', $mappings);
    $form_state->setRebuild(TRUE);
  }

  /**
   * Submit handler for removing a mapping.
   */
  public function removeMapping(array &$form, FormStateInterface $form_state) {
    $triggering_element = $form_state->getTriggeringElement();
    $index = substr($triggering_element['#name'], 7);
    $mappings = $form_state->get('mappings');
    unset($mappings[$index]);
    $form_state->set('mappings', array_values($mappings));
    $form_state->setRebuild(TRUE);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $selected_list = $form_state->getValue('lists');
    $this->config('brevo_contact_sync.settings')
      ->set('mappings', $form_state->getValue('mappings'))
      ->set('selected_list', $selected_list)
      ->save();
    parent::submitForm($form, $form_state);
  }
}
